// **********************************************************************
//                             Lode Runner
//                        ---------------------
//
//  File        : Block.h
//  Description : 
//  Author      : Guillaume Delhumeau <guillaume.delhumeau@gmail.com>
//  License     : GNU General Public License 3
//
// **********************************************************************

#ifndef _BLOCK_H_
#define _BLOCK_H_

#include <ClanLib/display.h>
#include "LevelResources.h"

class Block{

public:

	virtual void set_resources(LevelResources & resources) = 0;

	virtual bool is_wall() = 0;

	virtual bool is_breakable() = 0;

	virtual bool is_scale() = 0;

	virtual void draw(CL_GraphicContext & gc) = 0;

	inline unsigned int get_x() const { return _x; }
	inline unsigned int get_y() const { return _y; }
	inline void set_x(unsigned int x){ _x = x; }
	inline void set_y(unsigned int y){ _y = y; }
	inline void set_position(unsigned int x, unsigned int y){ _x = x; _y = y; }

private:
	unsigned int _x;

	unsigned int _y;

};

class EmptyBlock : public Block{

public:

	virtual void set_resources(LevelResources & resources){}

	virtual bool is_wall(){ return false; }

	virtual bool is_breakable(){ return false; }

	virtual bool is_scale(){ return false; }

	virtual void draw(CL_GraphicContext & gc){ };

};

class WallBlock : public Block{

public:

	virtual void set_resources(LevelResources & resources);

	virtual bool is_wall(){ return true; }

	virtual bool is_breakable(){ return false; }

	virtual bool is_scale(){ return false; }

	virtual void draw(CL_GraphicContext & gc);

private:

	CL_Sprite _sprite;

};

class WallBlock2 : public Block{

public:

	virtual void set_resources(LevelResources & resources);

	virtual bool is_wall(){ return true; }

	virtual bool is_breakable(){ return false; }

	virtual bool is_scale(){ return false; }

	virtual void draw(CL_GraphicContext & gc);

private:

	CL_Sprite _sprite;

};

class ScaleBlock : public Block{

public:

	virtual void set_resources(LevelResources & resources);

	virtual bool is_wall(){ return false; }

	virtual bool is_breakable(){ return false; }

	virtual bool is_scale(){ return true; }

	virtual void draw(CL_GraphicContext & gc);

private:

	CL_Sprite _sprite;

};

class BreakableWallBlock : public Block{

public:

	virtual void set_resources(LevelResources & resources);

	virtual bool is_wall(){ return true; }

	virtual bool is_breakable(){ return true; }

	virtual bool is_scale(){ return false; }

	virtual void draw(CL_GraphicContext & gc);

private:

	CL_Sprite _sprite;

};

class RopeBlock : public Block{

public:

	virtual void set_resources(LevelResources & resources);

	virtual bool is_wall(){ return false; }

	virtual bool is_breakable(){ return false; }

	virtual bool is_scale(){ return false; }

	virtual void draw(CL_GraphicContext & gc);

private:

	CL_Sprite _sprite;

};

class SecretPassageBlock : public Block{

public:

	virtual void set_resources(LevelResources & resources){}

	virtual bool is_wall(){ return false; }

	virtual bool is_breakable(){ return false; }

	virtual bool is_scale(){ return false; }

	virtual void draw(CL_GraphicContext & gc){ };

};

class FinalScaleBlock : public Block{

public:

	virtual void set_resources(LevelResources & resources){}

	virtual bool is_wall(){ return false; }

	virtual bool is_breakable(){ return false; }

	virtual bool is_scale(){ return false; }

	virtual void draw(CL_GraphicContext & gc){ };

};

class LootBlock : public Block{

public:

	virtual void set_resources(LevelResources & resources);

	virtual bool is_wall(){ return false; }

	virtual bool is_breakable(){ return false; }

	virtual bool is_scale(){ return false; }

	virtual void draw(CL_GraphicContext & gc);

private:

	CL_Sprite _sprite;

};


#endif