// **********************************************************************
//                             Lode Runner
//                        ---------------------
//
//  File        : Game.cpp
//  Description : 
//  Author      : Guillaume Delhumeau <guillaume.delhumeau@gmail.com>
//  License     : GNU General Public License 3
//
// **********************************************************************

#include <cstdlib>
#include "Game.h"
#include "Tools.h"

/************************************************************************/
/* Constructor                                                          */
/************************************************************************/
Game::Game(CL_GUIManager * p_manager, const CL_DisplayWindowDescription &desc):
CL_Window(p_manager, desc)
{
	func_close().set(this, &Game::_on_close);
	func_render().set(this, &Game::_on_render);		
	set_constant_repaint(true);

// 	button = new CL_PushButton(this);
// 	button->set_text("blah bli");
// 	CL_Rect client_area = get_client_area();
// 	button->set_geometry(CL_Rect(0, 0, 100, 100));

	_last_frame_time = 0;
	_game_modes_stack.push(&_level_editor_mode);
	_is_init = false;
}

/************************************************************************/
/* On render                                                            */
/************************************************************************/
void Game::_on_render(CL_GraphicContext &gc, const CL_Rect &clip_rect)
{
	if(!_is_init)
	{
		_init(gc);
	}

	unsigned int time = CL_System::get_time();
	_delta_time = (time - _last_frame_time) / 1000.0f;

	gc.clear();

	// Stretch the game to the window
	_zoom.scale = min((float) gc.get_width() / 800, 
		(float) gc.get_height() / 600);
	CL_Mat4f matrix_scale = CL_Mat4f::scale(_zoom.scale, _zoom.scale, 1.0f);

	// Center the game
	_zoom.dx = (int)(gc.get_width()-800*_zoom.scale)/2;
	_zoom.dy = (int)(gc.get_height()-600*_zoom.scale)/2;
	CL_Mat4f matrix_translate = CL_Mat4f::translate((float)_zoom.dx, (float)_zoom.dy, 0);

	// Apply transformations
	CL_Mat4f matrix_transform = matrix_scale * matrix_translate;
	gc.set_modelview(matrix_transform);

	if(!_game_modes_stack.empty())
	{
		IGameMode * p_game_mode = _game_modes_stack.top();

		p_game_mode->events(get_ic(), _zoom);
		p_game_mode->update(_delta_time);
		p_game_mode->draw(gc);
	}

	_last_frame_time = time;
}

/************************************************************************/
/* Init                                                                 */
/************************************************************************/
void Game::_init(CL_GraphicContext &gc)
{
	_level_editor_mode.load_graphics(gc);
	_is_init = true;
}

/************************************************************************/
/* On close                                                             */
/************************************************************************/
bool Game::_on_close()
{
	exit_with_code(EXIT_SUCCESS);
	return true;
}
