// **********************************************************************
//                             Lode Runner
//                        ---------------------
//
//  File        : SelectorMenu.cpp
//  Description : 
//  Author      : Guillaume Delhumeau <guillaume.delhumeau@gmail.com>
//  License     : GNU General Public License 3
//
// **********************************************************************

#include "SelectorMenu.h"
#include "Level.h"

/************************************************************************/
/* Constants                                                            */
/************************************************************************/
static const int APPEARING_SPEED = 140;
static const int MOUSE_SENSIBILITY_APPEARING = 10;

/************************************************************************/
/* Constructor                                                          */
/************************************************************************/
SelectorMenu::SelectorMenu(LevelResources & resources):
_resources(resources)
{
	_y = -MENU_HEIGHT;
	_state = STATE_HIDDEN;
	_selection = SELECTION_WALL;
	for(int i=0; i<NB_ITEMS; ++i)
	{
		_items_x[i] = 5 + i * (Level::BLOCK_SIZE + 10);
	}
}

/************************************************************************/
/* Draw                                                                 */
/************************************************************************/
void SelectorMenu::draw(CL_GraphicContext & gc)
{
	CL_Draw::fill(gc, 0, _y, 800, _y+MENU_HEIGHT, CL_Colorf::gray90);

	_resources.get_wall_sprite().draw(gc, _items_x[SELECTION_WALL], _y + 5.0f);
	_resources.get_wall2_sprite().draw(gc, _items_x[SELECTION_WALL_2], _y + 5.0f);
	_resources.get_scale_sprite().draw(gc, _items_x[SELECTION_SCALE], _y + 5.0f);
	_resources.get_rope_sprite().draw(gc, _items_x[SELECTION_ROPE], _y + 5.0f);
	_resources.get_breakable_sprite().draw(gc, _items_x[SELECTION_BREAKABLE], _y + 5.0f);
	_resources.get_loot_sprite().draw(gc, _items_x[SELECTION_LOOT], _y + 5.0f);
	_resources.get_end_sprite().draw(gc, _items_x[SELECTION_END], _y + 5.0f);

	CL_Draw::box(gc, _items_x[_selection], _y+5, 
		_items_x[_selection]+Level::BLOCK_SIZE, _y + 5 + Level::BLOCK_SIZE, CL_Colorf::red);

	CL_Draw::box(gc, _items_x[_selection]+1, _y+6, 
		_items_x[_selection]+Level::BLOCK_SIZE-1, _y + 4 + Level::BLOCK_SIZE, CL_Colorf::red);

	CL_Draw::fill(gc, 0, _y - MENU_HEIGHT, 800, 0, CL_Colorf::black);
}

void SelectorMenu::update(float delta_time)
{
	switch(_state)
	{
	case STATE_APPEARING:
		if(_y < 0)
		{
			_y += APPEARING_SPEED * delta_time;
			if(_y > 0)
			{
				_y = 0;
				_state = STATE_NORMAL;
			}
		}
		break;
	case STATE_NORMAL:
		break;
	case STATE_DISAPPEARING:
		if(_y > -MENU_HEIGHT)
		{
			_y -= APPEARING_SPEED * delta_time;
			if(_y < -MENU_HEIGHT)
			{
				_y = -MENU_HEIGHT;
				_state = STATE_HIDDEN;
			}
		}
		break;
	case STATE_HIDDEN:
		break;
	}
}

bool SelectorMenu::events(CL_InputContext & ic, ZoomParameters & zoom)
{
	if(ic.get_mouse_count()>0)
	{
		CL_InputDevice & mouse = ic.get_mouse();
		int mouse_y = (mouse.get_y()-zoom.dy) * zoom.scale;
		switch(_state)
		{
		case STATE_HIDDEN:
			if(mouse_y < MOUSE_SENSIBILITY_APPEARING)
			{
				_state = STATE_APPEARING;
			}
			break;
		case  STATE_NORMAL:
			if(mouse_y > MENU_HEIGHT + MOUSE_SENSIBILITY_APPEARING)
			{
				_state = STATE_DISAPPEARING;
			}
		default:
			break;			
		}

		if(_state == STATE_NORMAL && mouse.get_keycode(CL_MOUSE_LEFT) &&
			mouse_y < MENU_HEIGHT)
		{
			int mouse_x = (mouse.get_x() - zoom.dx) / zoom.scale;

			bool found = false;
			int selection = 0;
			do 
			{
				if(mouse_x >= _items_x[selection] && 
					mouse_x <= _items_x[selection] + Level::BLOCK_SIZE &&
					mouse_y >= 5 && 
					mouse_y <= 5 + Level::BLOCK_SIZE)
				{
					found = true;
				}				
				else
				{
					selection++;
				}
			} while (!found && selection < NB_ITEMS);
			
			switch(selection)
			{
			case SELECTION_WALL:
				_selection = SELECTION_WALL;
				break;
			case SELECTION_WALL_2:
				_selection = SELECTION_WALL_2;
				break;
			case SELECTION_SCALE:
				_selection = SELECTION_SCALE;
				break;
			case SELECTION_ROPE:
				_selection = SELECTION_ROPE;
				break;
			case SELECTION_BREAKABLE:
				_selection = SELECTION_BREAKABLE;
				break;
			case SELECTION_LOOT:
				_selection = SELECTION_LOOT;
				break;
			case SELECTION_END:
				_selection = SELECTION_END;
				break;
			}

			return true;
		}
	}

	return false;
}